@@ -1,7 +1,7 @@
 /*
     xattr - a python module for manipulating filesystem extended attributes
 
-    Copyright (C) 2002, 2003, 2006, 2008 Iustin Pop <iusty@k1024.org>
+    Copyright (C) 2002, 2003, 2006, 2008, 2012 Iustin Pop <iusty@k1024.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -34,13 +34,65 @@ typedef int Py_ssize_t;
 
 #if PY_MAJOR_VERSION >= 3
 #define IS_PY3K
+#define BYTES_CHAR "y"
 #else
+#define BYTES_CHAR "s"
 #define PyBytes_Check PyString_Check
 #define PyBytes_AS_STRING PyString_AS_STRING
 #define PyBytes_FromStringAndSize PyString_FromStringAndSize
 #define PyBytes_FromString PyString_FromString
 #endif
 
+#define ITEM_DOC \
+    ":param item: a string representing a file-name, or a file-like\n" \
+    "    object, or a file descriptor; this represents the file on \n" \
+    "    which to act\n"
+
+#define NOFOLLOW_DOC \
+    ":param nofollow: if true and if\n" \
+    "    the file name given is a symbolic link, the\n" \
+    "    function will operate on the symbolic link itself instead\n" \
+    "    of its target; defaults to false\n" \
+    ":type nofollow: boolean, optional\n" \
+
+#define NS_DOC \
+    ":param namespace: if given, the attribute must not contain the\n" \
+    "    namespace, but instead it will be taken from this parameter\n" \
+    ":type namespace: bytes\n"
+
+#define NAME_GET_DOC \
+    ":param string name: the attribute whose value to retrieve;\n" \
+    "    usually in the form of ``system.posix_acl`` or ``user.mime_type``\n"
+
+#define NAME_SET_DOC \
+    ":param string name: the attribute whose value to set;\n" \
+    "    usually in the form of ``system.posix_acl`` or ``user.mime_type``\n"
+
+#define NAME_REMOVE_DOC \
+    ":param string name: the attribute to remove;\n" \
+    "    usually in the form of ``system.posix_acl`` or \n" \
+    "    ``user.mime_type``\n"
+
+#define VALUE_DOC \
+    ":param string value: possibly with embedded NULLs; note that there\n" \
+    "    are restrictions regarding the size of the value, for\n" \
+    "    example, for ext2/ext3, maximum size is the block size\n" \
+
+#define FLAGS_DOC \
+    ":param flags: if 0 or omitted the attribute will be\n" \
+    "    created or replaced; if :const:`XATTR_CREATE`, the attribute\n" \
+    "    will be created, giving an error if it already exists;\n" \
+    "    if :const:`XATTR_REPLACE`, the attribute will be replaced,\n" \
+    "    giving an error if it doesn't exist;\n" \
+    ":type flags: integer\n"
+
+#define NS_CHANGED_DOC \
+    ".. versionchanged:: 0.5.1\n" \
+    "   The namespace argument, if passed, cannot be None anymore; to\n" \
+    "   explicitly specify an empty namespace, pass an empty\n" \
+    "   string (byte string under Python 3)."
+
+
 /* the estimated (startup) attribute buffer size in
    multi-operations */
 #define ESTIMATE_ATTR_SIZE 256
@@ -63,8 +115,29 @@ static void free_tgt(target_t *tgt) {
     }
 }
 
-/** Converts from a string, file or int argument to what we need. */
-static int convertObj(PyObject *myobj, target_t *tgt, int nofollow) {
+/* Used for cpychecker: */
+/* The checker automatically defines this preprocessor name when creating
+   the custom attribute: */
+#if defined(WITH_CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION_ATTRIBUTE)
+   #define CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION \
+__attribute__((cpychecker_negative_result_sets_exception))
+ #else
+   #define CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION
+ #endif
+
+static int convert_obj(PyObject *myobj, target_t *tgt, int nofollow)
+  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
+
+static int merge_ns(const char *ns, const char *name,
+                    const char **result, char **buf)
+  CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
+
+
+/** Converts from a string, file or int argument to what we need.
+ *
+ * Returns -1 on failure, 0 on success.
+ */
+static int convert_obj(PyObject *myobj, target_t *tgt, int nofollow) {
     int fd;
     tgt->tmp = NULL;
     if(PyBytes_Check(myobj)) {
@@ -76,40 +149,42 @@ static int convertObj(PyObject *myobj, target_t *tgt, int nofollow) {
             PyUnicode_AsEncodedString(myobj,
                                       Py_FileSystemDefaultEncoding, "strict");
         if(tgt->tmp == NULL)
-            return 0;
+            return -1;
         tgt->name = PyBytes_AS_STRING(tgt->tmp);
     } else if((fd = PyObject_AsFileDescriptor(myobj)) != -1) {
         tgt->type = T_FD;
         tgt->fd = fd;
     } else {
         PyErr_SetString(PyExc_TypeError, "argument must be string or int");
-        return 0;
+        return -1;
     }
-    return 1;
+    return 0;
 }
 
 /* Combine a namespace string and an attribute name into a
    fully-qualified name */
-static const char* merge_ns(const char *ns, const char *name, char **buf) {
-    if(ns != NULL) {
+static int merge_ns(const char *ns, const char *name,
+                    const char **result, char **buf) {
+    if(ns != NULL && *ns != '\0') {
         int cnt;
         size_t new_size = strlen(ns) + 1 + strlen(name) + 1;
         if((*buf = PyMem_Malloc(new_size)) == NULL) {
             PyErr_NoMemory();
-            return NULL;
+            return -1;
         }
         cnt = snprintf(*buf, new_size, "%s.%s", ns, name);
         if(cnt > new_size || cnt < 0) {
             PyErr_SetString(PyExc_ValueError,
                             "can't format the attribute name");
             PyMem_Free(*buf);
-            return NULL;
+            return -1;
         }
-        return *buf;
+        *result = *buf;
     } else {
         *buf = NULL;
-        return name;
+        *result = name;
     }
+    return 0;
 }
 
 static ssize_t _list_obj(target_t *tgt, char *list, size_t size) {
@@ -153,15 +228,16 @@ static int _remove_obj(target_t *tgt, const char *name) {
 /*
    Checks if an attribute name matches an optional namespace.
 
-   If the namespace is NULL, it will return the name itself.  If the
-   namespace is non-NULL and the name matches, it will return a
-   pointer to the offset in the name after the namespace and the
-   separator. If however the name doesn't match the namespace, it will
-   return NULL.
+   If the namespace is NULL or an empty string, it will return the
+   name itself.  If the namespace is non-NULL and the name matches, it
+   will return a pointer to the offset in the name after the namespace
+   and the separator. If however the name doesn't match the namespace,
+   it will return NULL.
+
 */
 const char *matches_ns(const char *ns, const char *name) {
     size_t ns_size;
-    if (ns == NULL)
+    if (ns == NULL || *ns == '\0')
         return name;
     ns_size = strlen(ns);
 
@@ -173,20 +249,16 @@ const char *matches_ns(const char *ns, const char *name) {
 
 /* Wrapper for getxattr */
 static char __pygetxattr_doc__[] =
+    "getxattr(item, attribute[, nofollow=False])\n"
     "Get the value of a given extended attribute (deprecated).\n"
     "\n"
-    "Parameters:\n"
-    "  - a string representing filename, or a file-like object,\n"
-    "    or a file descriptor; this represents the file on \n"
-    "    which to act\n"
-    "  - a string, representing the attribute whose value to retrieve;\n"
-    "    usually in form of system.posix_acl or user.mime_type\n"
-    "  - (optional) a boolean value (defaults to false), which, if\n"
-    "    the file name given is a symbolic link, makes the\n"
-    "    function operate on the symbolic link itself instead\n"
-    "    of its target;\n"
-    "@deprecated: since version 0.4, this function has been deprecated\n"
-    "    by the L{get} function\n"
+    ITEM_DOC
+    NAME_GET_DOC
+    NOFOLLOW_DOC
+    "\n"
+    ".. deprecated:: 0.4\n"
+    "   this function has been deprecated\n"
+    "   by the :func:`get` function.\n"
     ;
 
 static PyObject *
@@ -203,7 +275,7 @@ pygetxattr(PyObject *self, PyObject *args)
     /* Parse the arguments */
     if (!PyArg_ParseTuple(args, "Oet|i", &myarg, NULL, &attrname, &nofollow))
         return NULL;
-    if(!convertObj(myarg, &tgt, nofollow)) {
+    if(convert_obj(myarg, &tgt, nofollow) < 0) {
         res = NULL;
         goto freearg;
     }
@@ -243,6 +315,7 @@ pygetxattr(PyObject *self, PyObject *args)
 
 /* Wrapper for getxattr */
 static char __get_doc__[] =
+    "get(item, name[, nofollow=False, namespace=None])\n"
     "Get the value of a given extended attribute.\n"
     "\n"
     "Example:\n"
@@ -251,23 +324,16 @@ static char __get_doc__[] =
     "    >>> xattr.get('/path/to/file', 'comment', namespace=xattr.NS_USER)\n"
     "    'test'\n"
     "\n"
-    "@param item: the item to query; either a string representing the\n"
-    "    filename, or a file-like object, or a file descriptor\n"
-    "@param name: the attribute whose value to set; usually in form of\n"
-    "    system.posix_acl or user.mime_type\n"
-    "@type name: string\n"
-    "@param nofollow: if given and True, and the function is passed a\n"
-    "    filename that points to a symlink, the function will act on the\n"
-    "    symlink itself instead of its target\n"
-    "@type nofollow: boolean\n"
-    "@param namespace: if given, the attribute must not contain the\n"
-    "    namespace itself, but instead the namespace will be taken from\n"
-    "    this parameter\n"
-    "@type namespace: string\n"
-    "@return: the value of the extended attribute (can contain NULLs)\n"
-    "@rtype: string\n"
-    "@raise EnvironmentError: caused by any system errors\n"
-    "@since: 0.4\n"
+    ITEM_DOC
+    NAME_GET_DOC
+    NOFOLLOW_DOC
+    NS_DOC
+    ":return: the value of the extended attribute (can contain NULLs)\n"
+    ":rtype: string\n"
+    ":raises EnvironmentError: caused by any system errors\n"
+    "\n"
+    ".. versionadded:: 0.4\n"
+    NS_CHANGED_DOC
     ;
 
 static PyObject *
@@ -279,21 +345,24 @@ xattr_get(PyObject *self, PyObject *args, PyObject *keywds)
     char *attrname = NULL, *namebuf;
     const char *fullname;
     char *buf;
-    char *ns = NULL;
+    const char *ns = NULL;
     ssize_t nalloc, nret;
     PyObject *res;
     static char *kwlist[] = {"item", "name", "nofollow", "namespace", NULL};
 
     /* Parse the arguments */
-    if (!PyArg_ParseTupleAndKeywords(args, keywds, "Oet|iz", kwlist,
+    if (!PyArg_ParseTupleAndKeywords(args, keywds, "Oet|i" BYTES_CHAR, kwlist,
                                      &myarg, NULL, &attrname, &nofollow, &ns))
         return NULL;
-    if(!convertObj(myarg, &tgt, nofollow)) {
+    if(convert_obj(myarg, &tgt, nofollow) < 0) {
         res = NULL;
         goto freearg;
     }
 
-    fullname = merge_ns(ns, attrname, &namebuf);
+    if(merge_ns(ns, attrname, &fullname, &namebuf) < 0) {
+        res = NULL;
+        goto freearg;
+    }
 
     /* Find out the needed size of the buffer */
     if((nalloc = _get_obj(&tgt, fullname, NULL, 0)) == -1) {
@@ -332,60 +401,60 @@ xattr_get(PyObject *self, PyObject *args, PyObject *keywds)
 
 /* Wrapper for getxattr */
 static char __get_all_doc__[] =
+    "get_all(item[, nofollow=False, namespace=None])\n"
     "Get all the extended attributes of an item.\n"
     "\n"
     "This function performs a bulk-get of all extended attribute names\n"
     "and the corresponding value.\n"
     "Example:\n"
+    "\n"
     "    >>> xattr.get_all('/path/to/file')\n"
     "    [('user.mime-type', 'plain/text'), ('user.comment', 'test'),\n"
     "     ('system.posix_acl_access', '\\x02\\x00...')]\n"
     "    >>> xattr.get_all('/path/to/file', namespace=xattr.NS_USER)\n"
     "    [('mime-type', 'plain/text'), ('comment', 'test')]\n"
     "\n"
-    "@param item: the item to query; either a string representing the\n"
-    "    filename, or a file-like object, or a file descriptor\n"
-    "@keyword namespace: an optional namespace for filtering the\n"
-    "    attributes; for example, querying all user attributes can be\n"
-    "    accomplished by passing namespace=L{NS_USER}\n"
-    "@type namespace: string\n"
-    "@keyword nofollow: if passed and true, if the target file is a\n"
-    "    symbolic link, the attributes for the link itself will be\n"
-    "    returned, instead of the attributes of the target\n"
-    "@type nofollow: boolean\n"
-    "@return: list of tuples (name, value); note that if a namespace\n"
-    "    argument was passed, it (and the separator) will be stripped from\n"
-    "    the names returned\n"
-    "@rtype: list\n"
-    "@raise EnvironmentError: caused by any system errors\n"
-    "@note: Since reading the whole attribute list is not an atomic\n"
-    "    operation, it might be possible that attributes are added\n"
-    "    or removed between the initial query and the actual reading\n"
-    "    of the attributes; the returned list will contain only the\n"
-    "    attributes that were present at the initial listing of the\n"
-    "    attribute names and that were still present when the read\n"
-    "    attempt for the value is made.\n"
-    "@since: 0.4\n"
+    ITEM_DOC
+    ":keyword namespace: an optional namespace for filtering the\n"
+    "   attributes; for example, querying all user attributes can be\n"
+    "   accomplished by passing namespace=:const:`NS_USER`\n"
+    ":type namespace: string\n"
+    NOFOLLOW_DOC
+    ":return: list of tuples (name, value); note that if a namespace\n"
+    "   argument was passed, it (and the separator) will be stripped from\n"
+    "   the names returned\n"
+    ":rtype: list\n"
+    ":raises EnvironmentError: caused by any system errors\n"
+    "\n"
+    ".. note:: Since reading the whole attribute list is not an atomic\n"
+    "   operation, it might be possible that attributes are added\n"
+    "   or removed between the initial query and the actual reading\n"
+    "   of the attributes; the returned list will contain only the\n"
+    "   attributes that were present at the initial listing of the\n"
+    "   attribute names and that were still present when the read\n"
+    "   attempt for the value is made.\n"
+    ".. versionadded:: 0.4\n"
+    NS_CHANGED_DOC
     ;
 
 static PyObject *
 get_all(PyObject *self, PyObject *args, PyObject *keywds)
 {
     PyObject *myarg, *res;
-    int dolink=0;
-    char *ns = NULL;
+    int nofollow=0;
+    const char *ns = NULL;
     char *buf_list, *buf_val;
-    char *s;
+    const char *s;
     ssize_t nalloc, nlist, nval;
     PyObject *mylist;
     target_t tgt;
     static char *kwlist[] = {"item", "nofollow", "namespace", NULL};
 
     /* Parse the arguments */
-    if (!PyArg_ParseTupleAndKeywords(args, keywds, "O|iz", kwlist,
-                                     &myarg, &dolink, &ns))
+    if (!PyArg_ParseTupleAndKeywords(args, keywds, "O|i" BYTES_CHAR, kwlist,
+                                     &myarg, &nofollow, &ns))
         return NULL;
-    if(!convertObj(myarg, &tgt, dolink))
+    if(convert_obj(myarg, &tgt, nofollow) < 0)
         return NULL;
 
     /* Compute first the list of attributes */
@@ -414,6 +483,11 @@ get_all(PyObject *self, PyObject *args, PyObject *keywds)
 
     /* Create the list which will hold the result */
     mylist = PyList_New(0);
+    if(mylist == NULL) {
+      res = NULL;
+      goto free_buf_list;
+    }
+
     nalloc = ESTIMATE_ATTR_SIZE;
     if((buf_val = PyMem_Malloc(nalloc)) == NULL) {
         Py_DECREF(mylist);
@@ -438,7 +512,7 @@ get_all(PyObject *self, PyObject *args, PyObject *keywds)
                 if(errno == ERANGE) {
                     nval = _get_obj(&tgt, s, NULL, 0);
                     if((buf_val = PyMem_Realloc(buf_val, nval)) == NULL) {
-                        res = NULL;
+                        res = PyErr_NoMemory();
                         Py_DECREF(mylist);
                         goto free_buf_list;
                     }
@@ -450,6 +524,9 @@ get_all(PyObject *self, PyObject *args, PyObject *keywds)
                     missing = 1;
                     break;
                 }
+                /* else we're dealing with a different error, which we
+                   don't know how to handle nicely, so we abort */
+                Py_DECREF(mylist);
                 res = PyErr_SetFromErrno(PyExc_IOError);
                 goto freebufval;
             }
@@ -462,12 +539,16 @@ get_all(PyObject *self, PyObject *args, PyObject *keywds)
 #else
         my_tuple = Py_BuildValue("ss#", name, buf_val, nval);
 #endif
-
+        if (my_tuple == NULL) {
+          Py_DECREF(mylist);
+          res = NULL;
+          goto freebufval;
+        }
         PyList_Append(mylist, my_tuple);
         Py_DECREF(my_tuple);
     }
 
-    /* Successfull exit */
+    /* Successful exit */
     res = mylist;
 
  freebufval:
@@ -485,33 +566,23 @@ get_all(PyObject *self, PyObject *args, PyObject *keywds)
 
 
 static char __pysetxattr_doc__[] =
+    "setxattr(item, name, value[, flags=0, nofollow=False])\n"
     "Set the value of a given extended attribute (deprecated).\n"
     "\n"
-    "Be carefull in case you want to set attributes on symbolic\n"
+    "Be careful in case you want to set attributes on symbolic\n"
     "links, you have to use all the 5 parameters; use 0 for the \n"
-    "flags value if you want the default behavior (create or "
+    "flags value if you want the default behaviour (create or "
     "replace)\n"
     "\n"
-    "Parameters:\n"
-    "  - a string representing filename, or a file-like object,\n"
-    "    or a file descriptor; this represents the file on \n"
-    "    which to act\n"
-    "  - a string, representing the attribute whose value to set;\n"
-    "    usually in form of system.posix_acl or user.mime_type\n"
-    "  - a string, possibly with embedded NULLs; note that there\n"
-    "    are restrictions regarding the size of the value, for\n"
-    "    example, for ext2/ext3, maximum size is the block size\n"
-    "  - (optional) flags; if 0 or ommited the attribute will be \n"
-    "    created or replaced; if XATTR_CREATE, the attribute \n"
-    "    will be created, giving an error if it already exists;\n"
-    "    of XATTR_REPLACE, the attribute will be replaced,\n"
-    "    giving an error if it doesn't exists;\n"
-    "  - (optional) a boolean value (defaults to false), which, if\n"
-    "    the file name given is a symbolic link, makes the\n"
-    "    function operate on the symbolic link itself instead\n"
-    "    of its target;\n"
-    "@deprecated: since version 0.4, this function has been deprecated\n"
-    "    by the L{set} function\n"
+    ITEM_DOC
+    NAME_SET_DOC
+    VALUE_DOC
+    FLAGS_DOC
+    NOFOLLOW_DOC
+    "\n"
+    ".. deprecated:: 0.4\n"
+    "   this function has been deprecated\n"
+    "   by the :func:`set` function.\n"
     ;
 
 /* Wrapper for setxattr */
@@ -528,10 +599,10 @@ pysetxattr(PyObject *self, PyObject *args)
     target_t tgt;
 
     /* Parse the arguments */
-    if (!PyArg_ParseTuple(args, "Oetet#|bi", &myarg, NULL, &attrname,
+    if (!PyArg_ParseTuple(args, "Oetet#|ii", &myarg, NULL, &attrname,
                           NULL, &buf, &bufsize, &flags, &nofollow))
         return NULL;
-    if(!convertObj(myarg, &tgt, nofollow)) {
+    if(convert_obj(myarg, &tgt, nofollow) < 0) {
         res = NULL;
         goto freearg;
     }
@@ -558,39 +629,26 @@ pysetxattr(PyObject *self, PyObject *args)
 }
 
 static char __set_doc__[] =
+    "set(item, name, value[, flags=0, namespace=None])\n"
     "Set the value of a given extended attribute.\n"
     "\n"
     "Example:\n"
+    "\n"
     "    >>> xattr.set('/path/to/file', 'user.comment', 'test')\n"
     "    >>> xattr.set('/path/to/file', 'comment', 'test',"
     " namespace=xattr.NS_USER)\n"
     "\n"
-    "@param item: the item to query; either a string representing the\n"
-    "    filename, or a file-like object, or a file descriptor\n"
-    "@param name: the attribute whose value to set; usually in form of\n"
-    "    system.posix_acl or user.mime_type\n"
-    "@type name: string\n"
-    "@param value: a string, possibly with embedded NULLs; note that there\n"
-    "    are restrictions regarding the size of the value, for\n"
-    "    example, for ext2/ext3, maximum size is the block size\n"
-    "@type value: string\n"
-    "@param flags: if 0 or ommited the attribute will be\n"
-    "    created or replaced; if L{XATTR_CREATE}, the attribute\n"
-    "    will be created, giving an error if it already exists;\n"
-    "    if L{XATTR_REPLACE}, the attribute will be replaced,\n"
-    "    giving an error if it doesn't exists;\n"
-    "@type flags: integer\n"
-    "@param nofollow: if given and True, and the function is passed a\n"
-    "    filename that points to a symlink, the function will act on the\n"
-    "    symlink itself instead of its target\n"
-    "@type nofollow: boolean\n"
-    "@param namespace: if given, the attribute must not contain the\n"
-    "    namespace itself, but instead the namespace will be taken from\n"
-    "    this parameter\n"
-    "@type namespace: string\n"
-    "@rtype: None\n"
-    "@raise EnvironmentError: caused by any system errors\n"
-    "@since: 0.4\n"
+    ITEM_DOC
+    NAME_SET_DOC
+    VALUE_DOC
+    FLAGS_DOC
+    NOFOLLOW_DOC
+    NS_DOC
+    ":returns: None\n"
+    ":raises EnvironmentError: caused by any system errors\n"
+    "\n"
+    ".. versionadded:: 0.4\n"
+    NS_CHANGED_DOC
     ;
 
 /* Wrapper for setxattr */
@@ -605,29 +663,31 @@ xattr_set(PyObject *self, PyObject *args, PyObject *keywds)
     int nret;
     int flags = 0;
     target_t tgt;
-    char *ns = NULL;
+    const char *ns = NULL;
     char *newname;
     const char *full_name;
     static char *kwlist[] = {"item", "name", "value", "flags",
                              "nofollow", "namespace", NULL};
 
     /* Parse the arguments */
-    if (!PyArg_ParseTupleAndKeywords(args, keywds, "Oetet#|iiz", kwlist,
-                                     &myarg, NULL, &attrname, NULL,
+    if (!PyArg_ParseTupleAndKeywords(args, keywds, "Oetet#|ii" BYTES_CHAR,
+                                     kwlist, &myarg, NULL, &attrname, NULL,
                                      &buf, &bufsize, &flags, &nofollow, &ns))
         return NULL;
-    if(!convertObj(myarg, &tgt, nofollow)) {
+    if(convert_obj(myarg, &tgt, nofollow) < 0) {
         res = NULL;
         goto freearg;
     }
 
-    full_name = merge_ns(ns, attrname, &newname);
+    if(merge_ns(ns, attrname, &full_name, &newname) < 0) {
+        res = NULL;
+        goto freearg;
+    }
 
     /* Set the attribute's value */
     nret = _set_obj(&tgt, full_name, buf, bufsize, flags);
 
-    if(newname != NULL)
-        PyMem_Free(newname);
+    PyMem_Free(newname);
 
     free_tgt(&tgt);
 
@@ -649,21 +709,15 @@ xattr_set(PyObject *self, PyObject *args, PyObject *keywds)
 
 
 static char __pyremovexattr_doc__[] =
+    "removexattr(item, name[, nofollow])\n"
     "Remove an attribute from a file (deprecated).\n"
     "\n"
-    "Parameters:\n"
-    "  - a string representing filename, or a file-like object,\n"
-    "    or a file descriptor; this represents the file on \n"
-    "    which to act\n"
-    "  - a string, representing the attribute to be removed;\n"
-    "    usually in form of system.posix_acl or user.mime_type\n"
-    "  - (optional) a boolean value (defaults to false), which, if\n"
-    "    the file name given is a symbolic link, makes the\n"
-    "    function operate on the symbolic link itself instead\n"
-    "    of its target;\n"
-    "@deprecated: since version 0.4, this function has been deprecated\n"
-    "    by the L{remove}"
-    " function\n"
+    ITEM_DOC
+    NAME_REMOVE_DOC
+    NOFOLLOW_DOC
+    "\n"
+    ".. deprecated:: 0.4\n"
+    "   this function has been deprecated by the :func:`remove` function.\n"
     ;
 
 /* Wrapper for removexattr */
@@ -680,7 +734,7 @@ pyremovexattr(PyObject *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "Oet|i", &myarg, NULL, &attrname, &nofollow))
         return NULL;
 
-    if(!convertObj(myarg, &tgt, nofollow)) {
+    if(convert_obj(myarg, &tgt, nofollow) < 0) {
         res = NULL;
         goto freearg;
     }
@@ -706,27 +760,22 @@ pyremovexattr(PyObject *self, PyObject *args)
 }
 
 static char __remove_doc__[] =
+    "remove(item, name[, nofollow=False, namespace=None])\n"
     "Remove an attribute from a file.\n"
     "\n"
     "Example:\n"
+    "\n"
     "    >>> xattr.remove('/path/to/file', 'user.comment')\n"
     "\n"
-    "@param item: the item to query; either a string representing the\n"
-    "    filename, or a file-like object, or a file descriptor\n"
-    "@param name: the attribute whose value to set; usually in form of\n"
-    "    system.posix_acl or user.mime_type\n"
-    "@type name: string\n"
-    "@param nofollow: if given and True, and the function is passed a\n"
-    "    filename that points to a symlink, the function will act on the\n"
-    "    symlink itself instead of its target\n"
-    "@type nofollow: boolean\n"
-    "@param namespace: if given, the attribute must not contain the\n"
-    "    namespace itself, but instead the namespace will be taken from\n"
-    "    this parameter\n"
-    "@type namespace: string\n"
-    "@since: 0.4\n"
-    "@rtype: None\n"
-    "@raise EnvironmentError: caused by any system errors\n"
+    ITEM_DOC
+    NAME_REMOVE_DOC
+    NOFOLLOW_DOC
+    NS_DOC
+    ":returns: None\n"
+    ":raises EnvironmentError: caused by any system errors\n"
+    "\n"
+    ".. versionadded:: 0.4\n"
+    NS_CHANGED_DOC
     ;
 
 /* Wrapper for removexattr */
@@ -736,24 +785,23 @@ xattr_remove(PyObject *self, PyObject *args, PyObject *keywds)
     PyObject *myarg, *res;
     int nofollow = 0;
     char *attrname = NULL, *name_buf;
-    char *ns = NULL;
+    const char *ns = NULL;
     const char *full_name;
     int nret;
     target_t tgt;
     static char *kwlist[] = {"item", "name", "nofollow", "namespace", NULL};
 
     /* Parse the arguments */
-    if (!PyArg_ParseTupleAndKeywords(args, keywds, "Oet|iz", kwlist,
+    if (!PyArg_ParseTupleAndKeywords(args, keywds, "Oet|i" BYTES_CHAR, kwlist,
                                      &myarg, NULL, &attrname, &nofollow, &ns))
         return NULL;
 
-    if(!convertObj(myarg, &tgt, nofollow)) {
+    if(convert_obj(myarg, &tgt, nofollow) < 0) {
         res = NULL;
         goto freearg;
     }
 
-    full_name = merge_ns(ns, attrname, &name_buf);
-    if(full_name == NULL) {
+    if(merge_ns(ns, attrname, &full_name, &name_buf) < 0) {
         res = NULL;
         goto freearg;
     }
@@ -781,19 +829,14 @@ xattr_remove(PyObject *self, PyObject *args, PyObject *keywds)
 }
 
 static char __pylistxattr_doc__[] =
+    "listxattr(item[, nofollow=False])\n"
     "Return the list of attribute names for a file (deprecated).\n"
     "\n"
-    "Parameters:\n"
-    "  - a string representing filename, or a file-like object,\n"
-    "    or a file descriptor; this represents the file to \n"
-    "    be queried\n"
-    "  - (optional) a boolean value (defaults to false), which, if\n"
-    "    the file name given is a symbolic link, makes the\n"
-    "    function operate on the symbolic link itself instead\n"
-    "    of its target;\n"
-    "@deprecated: since version 0.4, this function has been deprecated\n"
-    "    by the L{list}"
-    " function\n"
+    ITEM_DOC
+    NOFOLLOW_DOC
+    "\n"
+    ".. deprecated:: 0.4\n"
+    "   this function has been deprecated by the :func:`list` function.\n"
     ;
 
 /* Wrapper for listxattr */
@@ -812,7 +855,7 @@ pylistxattr(PyObject *self, PyObject *args)
     /* Parse the arguments */
     if (!PyArg_ParseTuple(args, "O|i", &myarg, &nofollow))
         return NULL;
-    if(!convertObj(myarg, &tgt, nofollow))
+    if(convert_obj(myarg, &tgt, nofollow) < 0)
         return NULL;
 
     /* Find out the needed size of the buffer */
@@ -840,10 +883,18 @@ pylistxattr(PyObject *self, PyObject *args)
 
     /* Create the list which will hold the result */
     mylist = PyList_New(nattrs);
+    if(mylist == NULL)
+      goto freebuf;
 
     /* Create and insert the attributes as strings in the list */
     for(s = buf, nattrs = 0; s - buf < nret; s += strlen(s) + 1) {
-        PyList_SET_ITEM(mylist, nattrs, PyBytes_FromString(s));
+        PyObject *item = PyBytes_FromString(s);
+        if(item == NULL) {
+            Py_DECREF(mylist);
+            mylist = NULL;
+            goto freebuf;
+        }
+        PyList_SET_ITEM(mylist, nattrs, item);
         nattrs++;
     }
 
@@ -859,30 +910,28 @@ pylistxattr(PyObject *self, PyObject *args)
 }
 
 static char __list_doc__[] =
+    "list(item[, nofollow=False, namespace=None])\n"
     "Return the list of attribute names for a file.\n"
     "\n"
     "Example:\n"
+    "\n"
     "    >>> xattr.list('/path/to/file')\n"
     "    ['user.test', 'user.comment', 'system.posix_acl_access']\n"
     "    >>> xattr.list('/path/to/file', namespace=xattr.NS_USER)\n"
     "    ['test', 'comment']\n"
     "\n"
-    "@param item: the item to query; either a string representing the\n"
-    "    filename, or a file-like object, or a file descriptor\n"
-    "@param nofollow: if given and True, and the function is passed a\n"
-    "    filename that points to a symlink, the function will act on the\n"
-    "    symlink itself instead of its target\n"
-    "@type nofollow: boolean\n"
-    "@param namespace: if given, the attribute must not contain the\n"
-    "    namespace itself, but instead the namespace will be taken from\n"
-    "    this parameter\n"
-    "@type namespace: string\n"
-    "@return: list of strings; note that if a namespace argument was\n"
-    "    passed, it (and the separator) will be stripped from the names\n"
+    ITEM_DOC
+    NOFOLLOW_DOC
+    NS_DOC
+    ":returns: the list of attributes; note that if a namespace \n"
+    "    argument was passed, it (and the separator) will be stripped\n"
+    "    from the names\n"
     "    returned\n"
-    "@rtype: list\n"
-    "@raise EnvironmentError: caused by any system errors\n"
-    "@since: 0.4\n"
+    ":rtype: list\n"
+    ":raises EnvironmentError: caused by any system errors\n"
+    "\n"
+    ".. versionadded:: 0.4\n"
+    NS_CHANGED_DOC
     ;
 
 /* Wrapper for listxattr */
@@ -894,17 +943,17 @@ xattr_list(PyObject *self, PyObject *args, PyObject *keywds)
     ssize_t nalloc, nret;
     PyObject *myarg;
     PyObject *res;
-    char *ns = NULL;
+    const char *ns = NULL;
     Py_ssize_t nattrs;
     char *s;
     target_t tgt;
     static char *kwlist[] = {"item", "nofollow", "namespace", NULL};
 
     /* Parse the arguments */
-    if (!PyArg_ParseTupleAndKeywords(args, keywds, "O|iet", kwlist,
-                                     &myarg, &nofollow, NULL, &ns))
+    if (!PyArg_ParseTupleAndKeywords(args, keywds, "O|i" BYTES_CHAR, kwlist,
+                                     &myarg, &nofollow, &ns))
         return NULL;
-    if(!convertObj(myarg, &tgt, nofollow)) {
+    if(convert_obj(myarg, &tgt, nofollow) < 0) {
         res = NULL;
         goto freearg;
     }
@@ -934,12 +983,20 @@ xattr_list(PyObject *self, PyObject *args, PyObject *keywds)
     }
     /* Create the list which will hold the result */
     res = PyList_New(nattrs);
+    if(res == NULL)
+        goto freebuf;
 
     /* Create and insert the attributes as strings in the list */
     for(s = buf, nattrs = 0; s - buf < nret; s += strlen(s) + 1) {
         const char *name = matches_ns(ns, s);
         if(name!=NULL) {
-            PyList_SET_ITEM(res, nattrs, PyBytes_FromString(name));
+            PyObject *item = PyBytes_FromString(name);
+            if(item == NULL) {
+                Py_DECREF(res);
+                res = NULL;
+                goto freebuf;
+            }
+            PyList_SET_ITEM(res, nattrs, item);
             nattrs++;
         }
     }
@@ -951,7 +1008,6 @@ xattr_list(PyObject *self, PyObject *args, PyObject *keywds)
  freetgt:
     free_tgt(&tgt);
  freearg:
-    PyMem_Free(ns);
 
     /* Return the result */
     return res;
@@ -976,16 +1032,16 @@ static PyMethodDef xattr_methods[] = {
 };
 
 static char __xattr_doc__[] = \
-    "Interface to extended filesystem attributes.\n"
-    "\n"
     "This module gives access to the extended attributes present\n"
     "in some operating systems/filesystems. You can list attributes,\n"
     "get, set and remove them.\n"
     "\n"
     "The module exposes two sets of functions:\n"
-    "  - the 'old' L{listxattr}, L{getxattr}, L{setxattr}, L{removexattr}\n"
+    "  - the 'old' :func:`listxattr`, :func:`getxattr`, :func:`setxattr`,\n"
+    "    :func:`removexattr`\n"
     "    functions which are deprecated since version 0.4\n"
-    "  - the new L{list}, L{get}, L{get_all}, L{set}, L{remove} functions\n"
+    "  - the new :func:`list`, :func:`get`, :func:`get_all`, :func:`set`,\n"
+    "    :func:`remove` functions\n"
     "    which expose a namespace-aware API and simplify a bit the calling\n"
     "    model by using keyword arguments\n"
     "\n"
@@ -1001,23 +1057,23 @@ static char __xattr_doc__[] = \
     "  ['user.mime_type', 'user.comment']\n"
     "  >>> xattr.removexattr (\"file.txt\", \"user.comment\")\n"
     "\n"
-    "@note: Most or all errors reported by the system while using the xattr\n"
-    "library will be reported by raising a L{EnvironmentError}; under Linux,\n"
-    "the following C{errno} values are used:\n"
-    "  - C{ENOATTR} and C{ENODATA} mean that the attribute name is invalid\n"
-    "  - C{ENOTSUP} and C{EOPNOTSUPP} mean that the filesystem does not\n"
-    "    support extended attributes, or that the namespace is invalid\n"
-    "  - C{E2BIG} mean that the attribute value is too big\n"
-    "  - C{ERANGE} mean that the attribute name is too big (it might also\n"
-    "    mean an error in the xattr module itself)\n"
-    "  - C{ENOSPC} and C{EDQUOT} are documented as meaning out of disk space\n"
-    "    or out of disk space because of quota limits\n"
+    ".. note:: Most or all errors reported by the system while using\n"
+    "   the ``xattr`` library will be reported by raising\n"
+    "   a :exc:`EnvironmentError`; under\n"
+    "   Linux, the following ``errno`` values are used:\n"
+    "\n"
+    "   - ``ENOATTR`` and ``ENODATA`` mean that the attribute name is\n"
+    "     invalid\n"
+    "   - ``ENOTSUP`` and ``EOPNOTSUPP`` mean that the filesystem does not\n"
+    "     support extended attributes, or that the namespace is invalid\n"
+    "   - ``E2BIG`` mean that the attribute value is too big\n"
+    "   - ``ERANGE`` mean that the attribute name is too big (it might also\n"
+    "     mean an error in the xattr module itself)\n"
+    "   - ``ENOSPC`` and ``EDQUOT`` are documented as meaning out of disk\n"
+    "     space or out of disk space because of quota limits\n"
+    ".. note:: Under Python 3, the namespace argument is a byte string,\n"
+    "   not a unicode string.\n"
     "\n"
-    "@group Deprecated API: *xattr\n"
-    "@group Namespace constants: NS_*\n"
-    "@group set function flags: XATTR_CREATE, XATTR_REPLACE\n"
-    "@sort: list, get, get_all, set, remove, listxattr, getxattr, setxattr\n"
-    "    removexattr\n"
     ;
 
 #ifdef IS_PY3K
@@ -1041,6 +1097,10 @@ void
 initxattr(void)
 #endif
 {
+    PyObject *ns_security = NULL;
+    PyObject *ns_system   = NULL;
+    PyObject *ns_trusted  = NULL;
+    PyObject *ns_user     = NULL;
 #ifdef IS_PY3K
     PyObject *m = PyModule_Create(&xattrmodule);
 #else
@@ -1054,18 +1114,43 @@ initxattr(void)
     PyModule_AddStringConstant(m, "__version__", _XATTR_VERSION);
     PyModule_AddStringConstant(m, "__license__",
                                "GNU Lesser General Public License (LGPL)");
-    PyModule_AddStringConstant(m, "__docformat__", "epytext en");
+    PyModule_AddStringConstant(m, "__docformat__", "restructuredtext en");
 
     PyModule_AddIntConstant(m, "XATTR_CREATE", XATTR_CREATE);
     PyModule_AddIntConstant(m, "XATTR_REPLACE", XATTR_REPLACE);
 
     /* namespace constants */
-    PyModule_AddObject(m, "NS_SECURITY", PyBytes_FromString("security"));
-    PyModule_AddObject(m, "NS_SYSTEM", PyBytes_FromString("system"));
-    PyModule_AddObject(m, "NS_TRUSTED", PyBytes_FromString("trusted"));
-    PyModule_AddObject(m, "NS_USER", PyBytes_FromString("user"));
+    if((ns_security = PyBytes_FromString("security")) == NULL)
+        goto err_out;
+    if((ns_system = PyBytes_FromString("system")) == NULL)
+        goto err_out;
+    if((ns_trusted = PyBytes_FromString("trusted")) == NULL)
+        goto err_out;
+    if((ns_user = PyBytes_FromString("user")) == NULL)
+        goto err_out;
+    if(PyModule_AddObject(m, "NS_SECURITY", ns_security) < 0)
+        goto err_out;
+    ns_security = NULL;
+    if(PyModule_AddObject(m, "NS_SYSTEM", ns_system) < 0)
+        goto err_out;
+    ns_system = NULL;
+    if(PyModule_AddObject(m, "NS_TRUSTED", ns_trusted) < 0)
+        goto err_out;
+    ns_trusted = NULL;
+    if(PyModule_AddObject(m, "NS_USER", ns_user) < 0)
+        goto err_out;
+    ns_user = NULL;
 
 #ifdef IS_PY3K
     return m;
+#else
+    return;
 #endif
+
+ err_out:
+    Py_XDECREF(ns_user);
+    Py_XDECREF(ns_trusted);
+    Py_XDECREF(ns_system);
+    Py_XDECREF(ns_security);
+    INITERROR;
 }
